﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using AopAlliance.Intercept;
using Spring.Aop;
using System.Reflection;
using CodeCampService;
using System.Diagnostics;

namespace CustomSpringAspect
{
    public class LimitNumProductsAdvice : IMethodInterceptor
    {
        #region IMethodInterceptor Members
        public object Invoke(IMethodInvocation invocation)
        {
            object rval = invocation.Proceed();

            List<Product> products = rval as List<Product>;
            if (products != null && products.Count > 10)
            {
                products.RemoveRange(10, products.Count - 10);
                rval = products;
            }

            return rval;
        }

        #endregion
    }

    public class LogBeforeAdvice : IMethodBeforeAdvice
    {     

        #region IMethodBeforeAdvice Members

        public void Before(MethodInfo method, object[] args, object target)
        {
            string source = "Spring.Net AOP Advice";
            string log = "Application";
            string msg = string.Format("Entering Method {0}", method.Name);

            if (!EventLog.SourceExists(source))
                EventLog.CreateEventSource(source, log);

            EventLog.WriteEntry(source, msg);            

        }

        #endregion
    }
    
    public class LogAfterAdvice : IAfterReturningAdvice
    {
        #region IAfterReturningAdvice Members

        public void AfterReturning(object returnValue, MethodInfo method, object[] args, object target)
        {
            string source = "Spring.Net AOP Advice";
            string log = "Application";
            string msg = string.Format("Exiting Method {0}", method.Name);

            if (!EventLog.SourceExists(source))
                EventLog.CreateEventSource(source, log);

            EventLog.WriteEntry(source, msg);      
        }

        #endregion
    }

    public class CustomThrowingAdvice : IThrowsAdvice
    {
        public void AfterThrowing(MethodInfo method, object[] args, object target, Exception ex)
        {

        }
    }
}
