using System;
using System.Collections.Generic;
using System.Text;
using System.ServiceModel;
using System.ServiceModel.Description;
using System.ServiceModel.Dispatcher;
using System.ServiceModel.Configuration;
using Microsoft.Practices.EnterpriseLibrary.PolicyInjection;
using System.Reflection;

namespace PolicyInjectionBehaviors
{
   
    public class PolicyInjectionInstanceProvider : IInstanceProvider
    {
        private Type serviceContractType;
        
        public PolicyInjectionInstanceProvider(Type t)
        {            
            if (t!= null && !t.IsInterface)
            {
                throw new ArgumentException("Specified Type must be an interface");
            }

            this.serviceContractType = t;
        }

        #region IInstanceProvider Members

        public object GetInstance(InstanceContext instanceContext, System.ServiceModel.Channels.Message message)
        {
            Type type = instanceContext.Host.Description.ServiceType;

            if (serviceContractType != null)
            {
                MethodInfo method = null;
                foreach (MethodInfo m in typeof(PolicyInjection).GetMethods())
                {
                    if (m.Name == "Create" && m.GetParameters().Length == 1 && m.GetGenericArguments().Length == 2)
                    {
                        method = m;
                        break;
                    }
                }
                MethodInfo genericMethod = method.MakeGenericMethod(new Type[] { type, serviceContractType });
                return genericMethod.Invoke(null, new object[] { null });
            }
            else
            {
                if (!type.IsMarshalByRef)
                {
                    throw new ArgumentException("Type Must inherit MarshalByRefObject if no ServiceInterface is Specified");
                }
                MethodInfo method = null;
                foreach (MethodInfo m in typeof(PolicyInjection).GetMethods())
                {
                    if (m.Name == "Create" && m.GetParameters().Length == 1 && m.GetGenericArguments().Length == 1)
                    {
                        method = m;
                        break;
                    }
                }
                MethodInfo genericMethod = method.MakeGenericMethod(new Type[] { type });
                return genericMethod.Invoke(null, new object[] { null });
            }
        }

        public object GetInstance(InstanceContext instanceContext)
        {
            return GetInstance(instanceContext, null);
        }

        public void ReleaseInstance(InstanceContext instanceContext, object instance)
        {
            IDisposable disposable = instance as IDisposable;
            if (disposable != null)
            {
                disposable.Dispose();
            }

        }

        #endregion
    }

}
